window.Gleam.initSearch([{"doc":"gen_gleam","title":"gen_gleam","content":"# Generate Gleam\n\nGleam code generator in Gleam.\n\nSee tests for examples of usage.\n\n## Maturity\n\nNot generally usable.\n\nYet, used this code (with a few usages of `direct(String)`) to generate Gleam Code to solve the [Dec 19, 2023 puzzles of Advent of Code](https://adventofcode.com/2023/day/19), some 3000 resp. 4000 lines of Gleam for the 523 rules in my input.\n\n# TODO\n\n- [ ] extract some `OuterStatement` for imports, type defs and function defs, so we can generate a whole file, and eventually, multiple files\n- [ ] define types/functions for imports, so usage can, in principle, be type-checked\n- [ ] add functionality for `use`\n- [ ] add functionality for operators\n- [ ] add functionality for @external\n- [ ] add functionality for private types and functions\n- [ ] support for base types: string, tuple, ...\n- [ ] support for proper, nested pattern matching, e.g. `VariantPattern(&quot;Ok&quot;, [VariablePattern(&quot;x&quot;)])` even better if we get a `VariableStatement(&quot;x&quot;)` returned (prolly in hierarchy), that we can use in the subsequent statements.\n- [ ] add helpers prelude types and functions\n- [ ] add helpers stdlib types and functions\n- [ ] add functionality for ...\n\n## Done\n\n- [x] Construct a basic AST, supporting type definitions, case statements and function definitions\n- [x] generate Gleam code from that AST\n\n### Types\n- [x] Nil\n- [x] Int\n- [x] Variant type\n- [x] Parametrized type e.g. `a` in `List(a)`\n- [x] Function type\n\n### Pattern matching\n- [x] literal constants (bit blunt)\n- [x] let array\n- [x] let tuple\n\n### Statements\n- [x] instances of nil, int, variants, tuples\n- [x] case\n- [x] function\n\n### Prelude\n- [x] Boolean\n- [x] Result\n\n### StdLib\n- [x] Dict\n","url":"index.html"},{"doc":"gen_gleam/types","title":"Field","content":"pub type Field {\n  Field(name: String, field_type: GleamType)\n}\n\nField(name: String, field_type: GleamType)\n\n\nSynonyms:\ngen_gleam/types.Field\ngen_gleam/types Field","url":"gen_gleam/types.html#Field"},{"doc":"gen_gleam/types","title":"GleamType","content":"pub type GleamType {\n  AnonymousType(String)\n  NilType\n  IntType\n  ListType(list_type: GleamType)\n  TypeDefType(type_def: TypeDef)\n  FunctionType(\n    argument_types: List(GleamType),\n    result_type: GleamType,\n  )\n}\n\nAnonymousType(String)\n\n\nNilType\n\n\nIntType\n\n\nListType(list_type: GleamType)\n\n\nTypeDefType(type_def: TypeDef)\n\n\nFunctionType(\n  argument_types: List(GleamType),\n  result_type: GleamType,\n)\n\n\nSynonyms:\ngen_gleam/types.GleamType\ngen_gleam/types GleamType","url":"gen_gleam/types.html#GleamType"},{"doc":"gen_gleam/types","title":"TypeDef","content":"pub type TypeDef {\n  VariantType(name: String, variants: List(Variant))\n  VariantDepType(\n    name: String,\n    dep_types: List(GleamType),\n    variants: List(Variant),\n  )\n}\n\nVariantType(name: String, variants: List(Variant))\n\n\nVariantDepType(\n  name: String,\n  dep_types: List(GleamType),\n  variants: List(Variant),\n)\n\n\nSynonyms:\ngen_gleam/types.TypeDef\ngen_gleam/types TypeDef","url":"gen_gleam/types.html#TypeDef"},{"doc":"gen_gleam/types","title":"Variant","content":"pub type Variant {\n  Variant(name: String, fields: List(Field))\n}\n\nVariant(name: String, fields: List(Field))\n\n\nSynonyms:\ngen_gleam/types.Variant\ngen_gleam/types Variant","url":"gen_gleam/types.html#Variant"},{"doc":"gen_gleam/types","title":"anonymous","content":"pub fn anonymous(name: String) -&gt; GleamType\n\nSynonyms:\ngen_gleam/types.anonymous\ngen_gleam/types anonymous","url":"gen_gleam/types.html#anonymous"},{"doc":"gen_gleam/types","title":"dep_type_def","content":"pub fn dep_type_def(\n  name: String,\n  dep_types: List(GleamType),\n  variants: List(Variant),\n) -&gt; TypeDef\n Returns the `TypeDef` that you can pass to `generate_type_def`. If you want the type,\n use `variant_dep_type` instead.\nSynonyms:\ngen_gleam/types.dep_type_def\ngen_gleam/types dep_type_def","url":"gen_gleam/types.html#dep_type_def"},{"doc":"gen_gleam/types","title":"field","content":"pub fn field(name: String, field_type: GleamType) -&gt; Field\n\nSynonyms:\ngen_gleam/types.field\ngen_gleam/types field","url":"gen_gleam/types.html#field"},{"doc":"gen_gleam/types","title":"function","content":"pub fn function(\n  argument_types: List(GleamType),\n  result_type: GleamType,\n) -&gt; GleamType\n\nSynonyms:\ngen_gleam/types.function\ngen_gleam/types function","url":"gen_gleam/types.html#function"},{"doc":"gen_gleam/types","title":"generate_type","content":"pub fn generate_type(arg_type: GleamType) -&gt; String\n Generate the type when using the argument. To be used when you annotate any pattern matching.\n It is possible this will be done automatically from the statement, in some future version.\nSynonyms:\ngen_gleam/types.generate_type\ngen_gleam/types generate_type","url":"gen_gleam/types.html#generate_type"},{"doc":"gen_gleam/types","title":"generate_type_def","content":"pub fn generate_type_def(type_def: TypeDef) -&gt; String\n This is a root-statement\nSynonyms:\ngen_gleam/types.generate_type_def\ngen_gleam/types generate_type_def","url":"gen_gleam/types.html#generate_type_def"},{"doc":"gen_gleam/types","title":"int","content":"pub fn int() -&gt; GleamType\n\nSynonyms:\ngen_gleam/types.int\ngen_gleam/types int","url":"gen_gleam/types.html#int"},{"doc":"gen_gleam/types","title":"list","content":"pub fn list(item_type: GleamType) -&gt; GleamType\n\nSynonyms:\ngen_gleam/types.list\ngen_gleam/types list","url":"gen_gleam/types.html#list"},{"doc":"gen_gleam/types","title":"nil","content":"pub fn nil() -&gt; GleamType\n\nSynonyms:\ngen_gleam/types.nil\ngen_gleam/types nil","url":"gen_gleam/types.html#nil"},{"doc":"gen_gleam/types","title":"type_def","content":"pub fn type_def(name: String, variants: List(Variant)) -&gt; TypeDef\n Returns the `TypeDef` that you can pass to `generate_type_def`. If you want the type,\n use `variant_type` instead.\nSynonyms:\ngen_gleam/types.type_def\ngen_gleam/types type_def","url":"gen_gleam/types.html#type_def"},{"doc":"gen_gleam/types","title":"variant","content":"pub fn variant(name: String, fields: List(Field)) -&gt; Variant\n\nSynonyms:\ngen_gleam/types.variant\ngen_gleam/types variant","url":"gen_gleam/types.html#variant"},{"doc":"gen_gleam/types","title":"variant_dep_type","content":"pub fn variant_dep_type(\n  name: String,\n  dep_types: List(GleamType),\n  variants: List(Variant),\n) -&gt; GleamType\n For a type definition, the additional types are wildcards\n \n let a = anonymous(&quot;a&quot;)\n variant_dep_type(&quot;MyType&quot;, [a], [\n   variant(&quot;ListOfIt&quot;, [field(&quot;the_list&quot;, list(a))])\n ])\n |&gt; generate_type_def\n \n For usage, the additional types are concrete\n \n let t = Int\n variant_dep_type(&quot;MyType&quot;, [t], [\n   variant(&quot;ListOfIt&quot;, [field(&quot;the_list&quot;, list(t))])\n ])\n |&gt; generate_type\n \nSynonyms:\ngen_gleam/types.variant_dep_type\ngen_gleam/types variant_dep_type","url":"gen_gleam/types.html#variant_dep_type"},{"doc":"gen_gleam/types","title":"variant_type","content":"pub fn variant_type(\n  name: String,\n  variants: List(Variant),\n) -&gt; GleamType\n \n variant_type(&quot;MyType&quot;, [\n   variant(&quot;Bare&quot;, []).\n   variant(&quot;WithField&quot;, [field(&quot;number&quot;, int())])\n ])\n \nSynonyms:\ngen_gleam/types.variant_type\ngen_gleam/types variant_type","url":"gen_gleam/types.html#variant_type"},{"doc":"gen_gleam/types","title":"gen_gleam/types","content":" Provides built-in Gleam types. Provides means to define Gleam types (e.g. `variant_type`) and generate Gleam code for those (`generate_type_def`).","url":"gen_gleam/types.html"},{"doc":"gen_gleam/statements","title":"Argument","content":"pub opaque type Argument\n\n\nSynonyms:\ngen_gleam/statements.Argument\ngen_gleam/statements Argument","url":"gen_gleam/statements.html#Argument"},{"doc":"gen_gleam/statements","title":"Clause","content":"pub opaque type Clause\n\n\nSynonyms:\ngen_gleam/statements.Clause\ngen_gleam/statements Clause","url":"gen_gleam/statements.html#Clause"},{"doc":"gen_gleam/statements","title":"ComparisonOperator","content":"pub type ComparisonOperator {\n  Lt\n  Lte\n  Eq\n  Neq\n  Gt\n  Gte\n}\n\nLt\n\n\nLte\n\n\nEq\n\n\nNeq\n\n\nGt\n\n\nGte\n\n\nSynonyms:\ngen_gleam/statements.ComparisonOperator\ngen_gleam/statements ComparisonOperator","url":"gen_gleam/statements.html#ComparisonOperator"},{"doc":"gen_gleam/statements","title":"GleamStatement","content":"pub type GleamStatement {\n  NilVal\n  IntVal(value: Int)\n  ListVal(items: List(GleamStatement))\n  StringVal(string: String)\n  TupleVal(items: List(GleamStatement))\n  VariantVal(name: String, fields: List(GleamStatement))\n  VarPrimitive(name: String)\n  Comparison(GleamStatement, ComparisonOperator, GleamStatement)\n  CaseStatement(\n    guards: List(GleamStatement),\n    clauses: List(Clause),\n  )\n  Function(\n    name: String,\n    arguments: List(Argument),\n    statements: List(GleamStatement),\n  )\n  FunctionCall(name: String, arguments: List(GleamStatement))\n  UseArg(arg: GleamStatement)\n  LetVar(pattern: Pattern, statement: GleamStatement)\n  LetArray(patterns: List(Pattern), array: GleamStatement)\n  LetTuple(patterns: List(Pattern), tuple: GleamStatement)\n  Multiply(terms: List(GleamStatement))\n  Direct(string: String)\n}\n Might become opaque. Do use the functions to generate the constructors/variants.\nNilVal\n\n\nIntVal(value: Int)\n\n\nListVal(items: List(GleamStatement))\n\n\nStringVal(string: String)\n\n\nTupleVal(items: List(GleamStatement))\n\n\nVariantVal(name: String, fields: List(GleamStatement))\n\n\nVarPrimitive(name: String)\n\n\nComparison(GleamStatement, ComparisonOperator, GleamStatement)\n\n\nCaseStatement(\n  guards: List(GleamStatement),\n  clauses: List(Clause),\n)\n\n\nFunction(\n  name: String,\n  arguments: List(Argument),\n  statements: List(GleamStatement),\n)\n\n\nFunctionCall(name: String, arguments: List(GleamStatement))\n\n\nUseArg(arg: GleamStatement)\n\n\nLetVar(pattern: Pattern, statement: GleamStatement)\n\n\nLetArray(patterns: List(Pattern), array: GleamStatement)\n\n\nLetTuple(patterns: List(Pattern), tuple: GleamStatement)\n\n\nMultiply(terms: List(GleamStatement))\n\n\nDirect(string: String)\n\n\nSynonyms:\ngen_gleam/statements.GleamStatement\ngen_gleam/statements GleamStatement","url":"gen_gleam/statements.html#GleamStatement"},{"doc":"gen_gleam/statements","title":"Pattern","content":"pub opaque type Pattern\n\n\nSynonyms:\ngen_gleam/statements.Pattern\ngen_gleam/statements Pattern","url":"gen_gleam/statements.html#Pattern"},{"doc":"gen_gleam/statements","title":"arg_typed","content":"pub fn arg_typed(name: String, arg_type: GleamType) -&gt; Argument\n\nSynonyms:\ngen_gleam/statements.arg_typed\ngen_gleam/statements arg_typed","url":"gen_gleam/statements.html#arg_typed"},{"doc":"gen_gleam/statements","title":"arg_untyped","content":"pub fn arg_untyped(name: String) -&gt; Argument\n\nSynonyms:\ngen_gleam/statements.arg_untyped\ngen_gleam/statements arg_untyped","url":"gen_gleam/statements.html#arg_untyped"},{"doc":"gen_gleam/statements","title":"call","content":"pub fn call(\n  name: String,\n  arguments: List(GleamStatement),\n) -&gt; GleamStatement\n\nSynonyms:\ngen_gleam/statements.call\ngen_gleam/statements call","url":"gen_gleam/statements.html#call"},{"doc":"gen_gleam/statements","title":"case_stmt","content":"pub fn case_stmt(\n  guards: List(GleamStatement),\n  clauses: List(Clause),\n) -&gt; GleamStatement\n\nSynonyms:\ngen_gleam/statements.case_stmt\ngen_gleam/statements case_stmt","url":"gen_gleam/statements.html#case_stmt"},{"doc":"gen_gleam/statements","title":"clause","content":"pub fn clause(\n  patterns: List(Pattern),\n  stmt: GleamStatement,\n) -&gt; Clause\n\nSynonyms:\ngen_gleam/statements.clause\ngen_gleam/statements clause","url":"gen_gleam/statements.html#clause"},{"doc":"gen_gleam/statements","title":"compare","content":"pub fn compare(\n  stmt1: GleamStatement,\n  comp_op: ComparisonOperator,\n  stmt2: GleamStatement,\n) -&gt; GleamStatement\n\nSynonyms:\ngen_gleam/statements.compare\ngen_gleam/statements compare","url":"gen_gleam/statements.html#compare"},{"doc":"gen_gleam/statements","title":"direct","content":"pub fn direct(string: String) -&gt; GleamStatement\n @deprecated\nSynonyms:\ngen_gleam/statements.direct\ngen_gleam/statements direct","url":"gen_gleam/statements.html#direct"},{"doc":"gen_gleam/statements","title":"function","content":"pub fn function(\n  name: String,\n  arguments: List(Argument),\n  statements: List(GleamStatement),\n) -&gt; GleamStatement\n This is a root-statement\nSynonyms:\ngen_gleam/statements.function\ngen_gleam/statements function","url":"gen_gleam/statements.html#function"},{"doc":"gen_gleam/statements","title":"generate","content":"pub fn generate(stmt: GleamStatement) -&gt; String\n\nSynonyms:\ngen_gleam/statements.generate\ngen_gleam/statements generate","url":"gen_gleam/statements.html#generate"},{"doc":"gen_gleam/statements","title":"int","content":"pub fn int(value: Int) -&gt; GleamStatement\n\nSynonyms:\ngen_gleam/statements.int\ngen_gleam/statements int","url":"gen_gleam/statements.html#int"},{"doc":"gen_gleam/statements","title":"let_array","content":"pub fn let_array(\n  patterns: List(Pattern),\n  array: GleamStatement,\n) -&gt; GleamStatement\n TODO: Needs middle argument `Option(Pattern)` for `..tail`\nSynonyms:\ngen_gleam/statements.let_array\ngen_gleam/statements let_array","url":"gen_gleam/statements.html#let_array"},{"doc":"gen_gleam/statements","title":"let_tuple","content":"pub fn let_tuple(\n  patterns: List(Pattern),\n  tuple: GleamStatement,\n) -&gt; GleamStatement\n\nSynonyms:\ngen_gleam/statements.let_tuple\ngen_gleam/statements let_tuple","url":"gen_gleam/statements.html#let_tuple"},{"doc":"gen_gleam/statements","title":"let_var","content":"pub fn let_var(\n  pattern: Pattern,\n  statement: GleamStatement,\n) -&gt; GleamStatement\n\nSynonyms:\ngen_gleam/statements.let_var\ngen_gleam/statements let_var","url":"gen_gleam/statements.html#let_var"},{"doc":"gen_gleam/statements","title":"list","content":"pub fn list(values: List(GleamStatement)) -&gt; GleamStatement\n\nSynonyms:\ngen_gleam/statements.list\ngen_gleam/statements list","url":"gen_gleam/statements.html#list"},{"doc":"gen_gleam/statements","title":"literal","content":"pub fn literal(lit: String) -&gt; Pattern\n a constant pattern, such as `True` or `42` (represented as a string, for now)\n TODO make actual patterns for these constants\nSynonyms:\ngen_gleam/statements.literal\ngen_gleam/statements literal","url":"gen_gleam/statements.html#literal"},{"doc":"gen_gleam/statements","title":"multiply","content":"pub fn multiply(terms: List(GleamStatement)) -&gt; GleamStatement\n\nSynonyms:\ngen_gleam/statements.multiply\ngen_gleam/statements multiply","url":"gen_gleam/statements.html#multiply"},{"doc":"gen_gleam/statements","title":"name_of_var","content":"pub fn name_of_var(var: GleamStatement) -&gt; String\n\nSynonyms:\ngen_gleam/statements.name_of_var\ngen_gleam/statements name_of_var","url":"gen_gleam/statements.html#name_of_var"},{"doc":"gen_gleam/statements","title":"nil","content":"pub fn nil() -&gt; GleamStatement\n\nSynonyms:\ngen_gleam/statements.nil\ngen_gleam/statements nil","url":"gen_gleam/statements.html#nil"},{"doc":"gen_gleam/statements","title":"underscore","content":"pub fn underscore() -&gt; Pattern\n\nSynonyms:\ngen_gleam/statements.underscore\ngen_gleam/statements underscore","url":"gen_gleam/statements.html#underscore"},{"doc":"gen_gleam/statements","title":"unused_var","content":"pub fn unused_var(name: String) -&gt; Pattern\n name: variable name without the underscore, it will be prepended when generating code\nSynonyms:\ngen_gleam/statements.unused_var\ngen_gleam/statements unused_var","url":"gen_gleam/statements.html#unused_var"},{"doc":"gen_gleam/statements","title":"use_arg","content":"pub fn use_arg(arg: Argument) -&gt; GleamStatement\n\nSynonyms:\ngen_gleam/statements.use_arg\ngen_gleam/statements use_arg","url":"gen_gleam/statements.html#use_arg"},{"doc":"gen_gleam/statements","title":"var_pattern","content":"pub fn var_pattern(var: String) -&gt; Pattern\n\nSynonyms:\ngen_gleam/statements.var_pattern\ngen_gleam/statements var_pattern","url":"gen_gleam/statements.html#var_pattern"},{"doc":"gen_gleam/statements","title":"variable","content":"pub fn variable(name: String) -&gt; GleamStatement\n\nSynonyms:\ngen_gleam/statements.variable\ngen_gleam/statements variable","url":"gen_gleam/statements.html#variable"},{"doc":"gen_gleam/statements","title":"variant","content":"pub fn variant(\n  name: String,\n  field_values: List(GleamStatement),\n) -&gt; GleamStatement\n\nSynonyms:\ngen_gleam/statements.variant\ngen_gleam/statements variant","url":"gen_gleam/statements.html#variant"},{"doc":"gen_gleam/statements","title":"gen_gleam/statements","content":" For generating Gleam statements","url":"gen_gleam/statements.html"},{"doc":"gen_gleam/stdlib","title":"dict","content":"pub fn dict(\n  key_type: GleamType,\n  value_type: GleamType,\n) -&gt; GleamType\n Helper for `Dict` type\nSynonyms:\ngen_gleam/stdlib.dict\ngen_gleam/stdlib dict","url":"gen_gleam/stdlib.html#dict"},{"doc":"gen_gleam/stdlib","title":"gen_gleam/stdlib","content":" Helpers for types and functions in the `gleam_stdlib` package","url":"gen_gleam/stdlib.html"},{"doc":"gen_gleam/prelude","title":"boolean","content":"pub fn boolean() -&gt; GleamType\n Helper for `Boolean` type\nSynonyms:\ngen_gleam/prelude.boolean\ngen_gleam/prelude boolean","url":"gen_gleam/prelude.html#boolean"},{"doc":"gen_gleam/prelude","title":"error","content":"pub fn error(error: GleamStatement) -&gt; GleamStatement\n Helper for `Error(b)` statement\nSynonyms:\ngen_gleam/prelude.error\ngen_gleam/prelude error","url":"gen_gleam/prelude.html#error"},{"doc":"gen_gleam/prelude","title":"false","content":"pub fn false() -&gt; GleamStatement\n Helper for `False` statement\nSynonyms:\ngen_gleam/prelude.false\ngen_gleam/prelude false","url":"gen_gleam/prelude.html#false"},{"doc":"gen_gleam/prelude","title":"ok","content":"pub fn ok(value: GleamStatement) -&gt; GleamStatement\n Helper for `Ok(a)` statement\nSynonyms:\ngen_gleam/prelude.ok\ngen_gleam/prelude ok","url":"gen_gleam/prelude.html#ok"},{"doc":"gen_gleam/prelude","title":"result","content":"pub fn result(\n  ok_type: GleamType,\n  error_type: GleamType,\n) -&gt; GleamType\n Helper for `Result(a, b)` type\nSynonyms:\ngen_gleam/prelude.result\ngen_gleam/prelude result","url":"gen_gleam/prelude.html#result"},{"doc":"gen_gleam/prelude","title":"true","content":"pub fn true() -&gt; GleamStatement\n Helper for `True` value\nSynonyms:\ngen_gleam/prelude.true\ngen_gleam/prelude true","url":"gen_gleam/prelude.html#true"},{"doc":"gen_gleam/prelude","title":"gen_gleam/prelude","content":" Helpers for types and functions in the gleam prelude","url":"gen_gleam/prelude.html"}]);